local EventHandler = FlexBar2.OO.Mixin
{
	"PreClick_RegisterCastingEvent", -- Register casting events so the button is notified from casting or channeling (this way, only clicked buttons are notified and memory is spared so not all butons do the check)
	"PostClick_UncheckButton", -- Uncheck the default checking of the button after it has been clicked
	"PLAYER_ENTERING_WORLD_LoadButtonTextures", -- Update the button icons after all information is available to the UI
	"ACTIONBAR_UPDATE_COOLDOWN_SetSpinner", -- Set the cooldown spinner
	"ACTIONBAR_SLOT_CHANGED_UpdateTextures",
	"SpellCast_Start", -- Triggered whenever a spellcast starts (whenever a button is clicked)
	"SpellCast_End", -- Triggered whenever a spellcast ends, be it on the SPELLCAST_FAILED or SPELLCAST_STOP event
	"Channeling_Start", -- Triggered when the player starts to channel a spell
	"Channeling_Stop", -- Triggered when the player stops channeling a spell
	"StartDragging", -- Start dragging the button, may be triggered for various reasons,
	"StopDragging", -- Stop dragging the button, may be triggered for various reasons
	"DragActionStart",
	"DragActionStop",
}


function EventHandler:Activate()
	self:RegisterScript("PreClick", "PreClick_RegisterCastingEvent");
	self:RegisterScript("PostClick", "PostClick_UncheckButton");
	self:RegisterScript("OnReceiveDrag", "DragActionStop");
	self:RegisterEvent("ACTIONBAR_SLOT_CHANGED", "ACTIONBAR_SLOT_CHANGED_UpdateTextures");
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "PLAYER_ENTERING_WORLD_LoadButtonTextures");
	self:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN", "ACTIONBAR_UPDATE_COOLDOWN_SetSpinner");
end

function EventHandler:PLAYER_ENTERING_WORLD_LoadButtonTextures()
	self:LoadCurrentTexture();
end

function EventHandler:PreClick_RegisterCastingEvent()
	self:RegisterEvent("UNIT_SPELLCAST_START", "SpellCast_Start");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START","Channeling_Start");
end

function EventHandler:PostClick_UncheckButton()
	if(self.Check[1] ~= nil) then
		self.Frame:SetChecked(true);
	else
		self.Frame:SetChecked(false);
	end
end

function EventHandler:ACTIONBAR_SLOT_CHANGED_UpdateTextures(Event, Frame, Slot)
	-- Update actionbar slot texture if this button's slot corresponds with the changed slot(s)
	if(type(self.Settings) == "table" and type(self.Settings.Attributes) == "table" and self.Settings.Attributes.type == "action" and self.Settings.Attributes.action == Slot) then
		FlexBar2:TriggerEvent("ACTIONBAR_SLOT_CHANGED_UpdateTextures");
		self:LoadCurrentTexture();
	end
end

function EventHandler:ACTIONBAR_UPDATE_COOLDOWN_SetSpinner(Event, Frame, MouseButton)
	if(self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action") then
		local start, duration, enable = GetActionCooldown(self.Settings.Attributes.action);
		if(self.LastCooldown.start ~= start or self.LastCooldown.duration ~= duration) then
			self.LastCooldown.start = start;
			self.LastCooldown.duration = duration;
			if(FlexBar2.db.profile.Ticker == "global") then
				if(duration <= 1.5) then
					CooldownFrame_SetTimer(self.Frame.Cooldown, start, duration, enable);
				end
			elseif(FlexBar2.db.profile.Ticker == "noglobal") then
				if(duration > 1.5) then
					CooldownFrame_SetTimer(self.Frame.Cooldown, start, duration, enable);
				end
			elseif(FlexBar2.db.profile.Ticker == true) then
				CooldownFrame_SetTimer(self.Frame.Cooldown, start, duration, enable);
			end
		end
	end
end

function EventHandler:SpellCast_Start(Event, Frame, Unit)
	if(Unit == "player" and self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action" and IsCurrentAction(self.Settings.Attributes.action)) then
		FlexBar2:TriggerEvent("SpellCast_Start");
		self:UnregisterEvent("UNIT_SPELLCAST_START", "SpellCast_Start");
		self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START","Channeling_Start");
		self:RegisterEvent("UNIT_SPELLCAST_STOP", "SpellCast_End");
	end
end

function EventHandler:SpellCast_End(Event, Frame, Unit)
	if(Unit == "player" and self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action") then
		FlexBar2:TriggerEvent("SpellCast_End");
		self:UnregisterEvent("UNIT_SPELLCAST_STOP", "SpellCast_End");
	end
end

function EventHandler:Channeling_Start(Event, Frame, Unit)
	if(Unit == "player" and self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action" and IsCurrentAction(self.Settings.Attributes.action)) then
		FlexBar2:TriggerEvent("Channeling_Start");
		self:UnregisterEvent("UNIT_SPELLCAST_START", "SpellCast_Start");
		self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START","Channeling_Start");
		self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP", "Channeling_Stop");
	end
end

function EventHandler:Channeling_Stop(Event, Frame, Unit)
	if(Unit == "player" and self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action") then
		FlexBar2:TriggerEvent("Channeling_Stop");
		self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP", "Channeling_Stop");
	end
end


function EventHandler:StartDragging(Event, Frame, MouseButton)
	if(MouseButton == "LeftButton") then
		self.Frame:StartMoving();
		self.IsMoving = true;
	end
end

function EventHandler:StopDragging()
	local _ ,_ ,_ ,X ,Y = self.Frame:GetPoint();
	if(self.IsMoving == true) then
		self.Frame:StopMovingOrSizing();
		self.IsMoving = false;
		self.Settings.X = X;
		self.Settings.Y = -Y;
	end
	return X, Y;
end

function EventHandler:DragActionStart()
	if(self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action" and HasAction(self.Settings.Attributes.action)) then
		FlexBar2:TriggerEvent("FB2_BUTTON_CHANGED");
		PickupAction(self.Settings.Attributes.action);
	end
end

function EventHandler:DragActionStop()
	if(self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action") then
		FlexBar2:TriggerEvent("FB2_BUTTON_CHANGED");
		PlaceAction(self.Settings.Attributes.action);
	end
end

FlexBar2:RegisterHandler(EventHandler);

FlexBar2.Group:RegisterHandler("StartDragging", function(self, Event, Frame, MouseButton) -- Handler for group dragging
	if(MouseButton == "LeftButton") then
		FlexBar2:Debug("StartDragging");
		_, _, _, pX, pY = self.Frame:GetPoint();
		for _, Member in ipairs(self.Group) do
			if(Member ~= self) then -- Ignore this frame or we'll end up anchor it to itself
				local _, _, _, cX, cY = Member.Frame:GetPoint();
				FlexBar2:Print(cX, cY);
				Member.Frame:ClearAllPoints();
				Member.Frame:SetPoint("TOPLEFT", self.Frame, "TOPLEFT", (cX - pX), (cY - pY));
				Member.IsMoving = true;
				FlexBar2:Debug("Anchoring " , Member.Button, " to ", self.Button);
			end
		end
		self:StartDragging(Event, self.Frame, MouseButton);
	end
end);

FlexBar2.Group:RegisterHandler("StopDragging", function(self, Event, Frame)
	local pX, pY = self:StopDragging();
	for _, Member in ipairs(self.Group) do
		if(Member ~= self) then
			_, _, _, cX, cY = Member.Frame:GetPoint();
			Member.Frame:ClearAllPoints();
			Member.Frame:SetPoint("TOPLEFT", "UIParent", "TOPLEFT", (pX + cX), (pY + cY));
			Member:StopDragging();
		end
	end
end);

FlexBar2.Group:RegisterHandler("HorizontalGroup", function(self)
	for Key, Member in ipairs(self.Group) do
		Member:SetPosition(100 + Key * 38, 100 ,true);
	end
end);

FlexBar2.Group:RegisterHandler("VerticalGroup", function(self)
	for Key, Member in ipairs(self.Group) do
		Member:SetPosition(100, 100 + Key * 38 ,true);
	end
end);

FlexBar2.Group:RegisterHandler("SquareGroup", function(self, Rows)
	for Key, Member in ipairs(self.Group) do
		Member:SetPosition(100 + (38 * (Key - ((math.ceil(Key / Rows) - 1) * Rows))), 100 + (38 * (math.ceil(Key / Rows))) ,true);
	end
end);
